@@ -13,11 +13,15 @@ temporal dynamics.
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 """
 
+from __future__ import annotations
+
 import json
+import logging
 from typing import List  # @UnusedImport
 from typing import (
     TYPE_CHECKING,
     Any,
+    Callable,
     Dict,
     Generator,
     Iterator,
@@ -31,11 +35,12 @@ import numpy as np
 
 from pde.fields import ScalarField
 from pde.grids.base import GridBase
-from pde.grids.cartesian import CartesianGridBase
+from pde.grids.cartesian import CartesianGrid
 from pde.storage.base import StorageBase
 from pde.tools.cuboid import Cuboid
-from pde.tools.misc import display_progress
-from pde.tools.plotting import plot_on_axes
+from pde.tools.docstrings import fill_in_docstring
+from pde.tools.output import display_progress
+from pde.tools.plotting import PlotReference, plot_on_axes
 from pde.trackers.base import InfoDict
 from pde.trackers.intervals import IntervalType
 
@@ -49,7 +54,9 @@ DropletSequence = Union[Generator, Sequence[SphericalDroplet]]
 
 
 class Emulsion(list):
-    """ class representing a collection of droplets in a common system """
+    """class representing a collection of droplets in a common system"""
+
+    _show_projection_warning: bool = True
 
     def __init__(
         self,
@@ -104,7 +111,7 @@ class Emulsion(list):
         # return sum of Emulsions as an emulsion
         return Emulsion(list.__add__(self, rhs))
 
-    def __getitem__(self, key: Union[int, slice]):
+    def __getitem__(self, key: Union[int, slice]):  # type: ignore
         # return result from extended slicing as Emulsion
         result = list.__getitem__(self, key)
         try:
@@ -112,7 +119,7 @@ class Emulsion(list):
         except TypeError:
             return result
 
-    def copy(self, min_radius: float = -1) -> "Emulsion":
+    def copy(self, min_radius: float = -1) -> Emulsion:
         """return a copy of this emulsion
 
         Args:
@@ -132,7 +139,7 @@ class Emulsion(list):
         self,
         droplets: DropletSequence,
         copy: bool = True,
-    ):
+    ) -> None:
         """add many droplets to the emulsion
 
         Args:
@@ -142,28 +149,29 @@ class Emulsion(list):
         for droplet in droplets:
             self.append(droplet, copy=copy)
 
-    def append(self, droplet: SphericalDroplet, copy: bool = True):
+    def append(self, droplet: SphericalDroplet, copy: bool = True) -> None:
         """add a droplet to the emulsion
 
         Args:
-            droplet (SphericalDroplet): Droplet to add to the emulsion
-            copy (bool, optional): Whether to make a copy of the droplet or not
+            droplet (:class:`droplets.dropelts.SphericalDroplet`):
+                Droplet to add to the emulsion
+            copy (bool, optional):
+                Whether to make a copy of the droplet or not
         """
         if self.dim is None:
             self.dim = droplet.dim
         elif self.dim != droplet.dim:
             raise ValueError(
-                "Cannot append droplet in dimension "
-                f"{droplet.dim} to emulsion in dimension "
-                f"{self.dim}"
+                f"Cannot append droplet in dimension {droplet.dim} to emulsion in "
+                f"dimension {self.dim}"
             )
         if copy:
             droplet = droplet.copy()
         super().append(droplet)
 
     @property
-    def data(self):
-        """numpy.ndarray: an array containing the data of the full emulsion
+    def data(self) -> Optional[np.ndarray]:
+        """:class:`~numpy.ndarray`: an array containing the data of the full emulsion
 
         This requires all droplets to be of the same class
         """
@@ -184,12 +192,13 @@ class Emulsion(list):
         """link the data of all droplets in a single array
 
         Returns:
-            numpy.ndarray: The array containing all droplet data. If entries in
+            :class:`~numpy.ndarray`: The array containing all droplet data. If entries in
                 this array are modified, it will be reflected in the droplets.
         """
         if len(self) == 0:
             data = np.empty(0)
         else:
+            assert self.data is not None
             data = self.data  # create an array with all the droplet data
             # link back to droplets
             for i, d in enumerate(self):
@@ -197,13 +206,15 @@ class Emulsion(list):
         return data
 
     @classmethod
-    def _from_hdf_dataset(cls, dataset, grid: Optional[GridBase] = None) -> "Emulsion":
+    def _from_hdf_dataset(cls, dataset, grid: Optional[GridBase] = None) -> Emulsion:
         """construct an emulsion by reading data from an hdf5 dataset
 
         Args:
-            dataset: an HDF5 dataset from which the data of the emulsion is read
-            grid (GridBase): The grid on which the droplets are defined. This
-                information is required to measure distances between droplets.
+            dataset:
+                an HDF5 dataset from which the data of the emulsion is read
+            grid (:class:`pde.grids.base.GridBase`):
+                The grid on which the droplets are defined. This information is required
+                to measure distances between droplets.
         """
         # there are values, so the emulsion is not empty
         droplet_class = dataset.attrs["droplet_class"]
@@ -217,17 +228,19 @@ class Emulsion(list):
         return cls(droplets, grid=grid, copy=False)
 
     @classmethod
-    def from_file(cls, filename: str) -> "Emulsion":
+    def from_file(cls, path: str) -> Emulsion:
         """create emulsion by reading file
 
         Args:
-            filename (str): Name of the file to read emulsion from
+            path (str):
+                The path from which the data is read. This function assumes that the
+                data was written as an HDF5 file using :meth:`to_file`.
         """
         import h5py
 
-        with h5py.File(filename, "r") as fp:
+        with h5py.File(path, "r") as fp:
             if len(fp) != 1:
-                raise RuntimeError(f"Multiple emulsions found in file `{filename}`")
+                raise RuntimeError(f"Multiple emulsions found in file `{path}`")
 
             # read grid
             if "grid" in fp.attrs:
@@ -242,7 +255,7 @@ class Emulsion(list):
         return obj
 
     def _write_hdf_dataset(self, hdf_path, key: str = "emulsion"):
-        """ write data to a given hdf5 path `hdf_path` """
+        """write data to a given hdf5 path `hdf_path`"""
         if self:
             # emulsion contains at least one droplet
             dataset = hdf_path.create_dataset(key, data=self.data)
@@ -256,15 +269,18 @@ class Emulsion(list):
 
         return dataset
 
-    def to_file(self, filename: str):
+    def to_file(self, path: str) -> None:
         """store data in hdf5 file
 
+        The data can be read using the classmethod :meth:`Emulsion.from_file`.
+
         Args:
-            filename (str): Name of the file to write emulsion to
+            path (str):
+                The path to which the data is written as an HDF5 file.
         """
         import h5py
 
-        with h5py.File(filename, "w") as fp:
+        with h5py.File(path, "w") as fp:
             # write the grid data
             if self.grid is not None:
                 fp.attrs["grid"] = self.grid.state_serialized
@@ -298,7 +314,7 @@ class Emulsion(list):
 
     @property
     def bbox(self) -> Cuboid:
-        """ Cuboid: bounding box of the emulsion """
+        """:class:`Cuboid`: bounding box of the emulsion"""
         if len(self) == 0:
             raise RuntimeError("Bounding box of empty emulsion is undefined")
         return sum((droplet.bbox for droplet in self[1:]), self[0].bbox)
@@ -316,7 +332,7 @@ class Emulsion(list):
                 Optional label for the returned scalar field
 
         Returns:
-            :class:`pde.fields.scalar.ScalarField`: the actual phase field
+            :class:`~pde.fields.scalar.ScalarField`: the actual phase field
         """
         if grid is None:
             grid = self.grid
@@ -329,11 +345,11 @@ class Emulsion(list):
         else:
             result: ScalarField = self[0].get_phase_field(grid, label=label)
             for d in self[1:]:
-                result += d.get_phase_field(grid)  # type: ignore
+                result += d.get_phase_field(grid)
             np.clip(result.data, 0, 1, out=result.data)
             return result
 
-    def remove_small(self, min_radius: float = -np.inf):
+    def remove_small(self, min_radius: float = -np.inf) -> None:
         """remove droplets that are very small
 
         The emulsions is modified in-place.
@@ -348,7 +364,7 @@ class Emulsion(list):
             if self[i].radius <= min_radius:
                 self.pop(i)
 
-    def get_pairwise_distances(self, subtract_radius: bool = False):
+    def get_pairwise_distances(self, subtract_radius: bool = False) -> np.ndarray:
         """return the pairwise distance between droplets
 
         Args:
@@ -357,12 +373,12 @@ class Emulsion(list):
                 center.
 
         Returns:
-            np.array: a matrix with the distances between all droplets
+            :class:`~numpy.ndarray`: a matrix with the distances between all droplets
         """
         if self.grid is None:
 
             def get_distance(p1, p2):
-                """ helper function calculating the distance between points """
+                """helper function calculating the distance between points"""
                 return np.linalg.norm(p1 - p2)
 
         else:
@@ -382,7 +398,7 @@ class Emulsion(list):
 
         return dists
 
-    def get_neighbor_distances(self, subtract_radius: bool = False):
+    def get_neighbor_distances(self, subtract_radius: bool = False) -> np.ndarray:
         """calculates the distance of each droplet to its nearest neighbor
 
         Warning: Nearest neighbors are defined by comparing the distances
@@ -394,7 +410,7 @@ class Emulsion(list):
                 the surfaces instead of the positions
 
         Returns:
-            np.array: a vector with a distance for each droplet
+            :class:`~numpy.ndarray`: a vector with a distance for each droplet
         """
         # handle simple cases
         if len(self) == 0:
@@ -408,16 +424,17 @@ class Emulsion(list):
             from scipy.spatial import KDTree
 
         # build tree to query the nearest neighbors
+        assert self.data is not None
         positions = self.data["position"]
         tree = KDTree(positions)
         dist, index = tree.query(positions, 2)
 
         if subtract_radius:
-            return dist[:, 1] - self.data["radius"][index].sum(axis=1)
+            return dist[:, 1] - self.data["radius"][index].sum(axis=1)  # type: ignore
         else:
-            return dist[:, 1]
+            return dist[:, 1]  # type: ignore
 
-    def remove_overlapping(self, min_distance: float = 0):
+    def remove_overlapping(self, min_distance: float = 0) -> None:
         """remove all droplets that are overlapping.
 
         If a pair of overlapping droplets was found, the smaller one of these
@@ -439,7 +456,7 @@ class Emulsion(list):
             x, y = np.unravel_index(np.argmin(dists), dists.shape)
             if dists[x, y] < min_distance:
                 # droplets overlap -> remove the smaller one
-                if self[x].radius > self[y].radius:
+                if self[x].radius > self[y].radius:  # type: ignore
                     self.pop(y)
                     dists = np.delete(np.delete(dists, y, 0), y, 1)
                 else:
@@ -450,7 +467,7 @@ class Emulsion(list):
 
     @property
     def total_droplet_volume(self) -> float:
-        """ float: the total volume of all droplets """
+        """float: the total volume of all droplets"""
         return sum(droplet.volume for droplet in self)
 
     def get_size_statistics(self) -> Dict[str, float]:
@@ -485,13 +502,19 @@ class Emulsion(list):
         field: ScalarField = None,
         image_args: Dict[str, Any] = None,
         repeat_periodically: bool = True,
+        color_value: Callable = None,
+        cmap=None,
+        norm=None,
+        colorbar: Union[bool, str] = True,
         **kwargs,
-    ):
+    ) -> PlotReference:
         """plot the current emulsion together with a corresponding field
 
+        If the emulsion is defined in a 3d geometry, only a projection on the first two
+        axes is shown.
+
         Args:
-            ax (:class:`matplotlib.axes.Axes`):
-                The axes in which the background is shown
+            {PLOT_ARGS}
             field (:class:`pde.fields.scalar.ScalarField`):
                 provides the phase field that is shown as a background
             image_args (dict):
@@ -501,15 +524,38 @@ class Emulsion(list):
             repeat_periodically (bool):
                 flag determining whether droplets are shown on both sides of
                 periodic boundary conditions. This option can slow down plotting
+            color_value (callable):
+                Function used to determine the color of a droplet. The function is
+                called with individual droplet objects and must return a single scalar
+                value, which is then mapped to a color using the colormap given by
+                `cmap` and a suitable normalization given by `norm`.
+            cmap (str or :class:`~matplotlib.colors.Colormap`):
+                The colormap used to map normalized data values to RGBA colors.
+            norm (:class:`~matplotlib.colors.Normalize`):
+                The normalizing object which scales data, typically into the interval
+                [0, 1]. If None, norm defaults to a `colors.Normalize` object which
+                maps the range of values obtained from `color_value` to [0, 1].
+            colorbar (bool or str):
+                Determines whether a colorbar is shown when `color_value` is supplied.
+                If a string is given, it is used as a label for the colorbar.
             **kwargs:
                 Additional keyword arguments are passed to the function creating the
                 patch that represents the droplet. For instance, to only draw the
                 outlines of the droplets, you may need to supply `fill=False`.
+
+        Returns:
+            :class:`~pde.tools.plotting.PlotReference`: Information about the plot
         """
-        if self.dim != 2:
+        if self.dim is None or self.dim <= 1:
             raise NotImplementedError(
                 f"Plotting emulsions in {self.dim} dimensions is not implemented."
             )
+        elif self.dim > 2:
+            if Emulsion._show_projection_warning:
+                logger = logging.getLogger(self.__class__.__name__)
+                logger.warning("A projection on the first two axes is shown.")
+                Emulsion._show_projection_warning = False
+
         grid_compatible = (
             self.grid is None or field is None or self.grid.compatible_with(field.grid)
         )
@@ -528,7 +574,7 @@ class Emulsion(list):
                 grid = field.grid
 
         else:
-            if isinstance(grid, CartesianGridBase):
+            if isinstance(grid, CartesianGrid):
                 # determine the bounds from the (2d) grid
                 bounds = grid.axes_bounds
             else:
@@ -538,20 +584,41 @@ class Emulsion(list):
             ax.set_ylim(*bounds[1])
             ax.set_aspect("equal")
 
+        # determine the color of all droplets
+        if color_value is not None:
+            import matplotlib.pyplot as plt
+
+            # determine the scalar values associated with all droplets
+            values = np.array([color_value(droplet) for droplet in self])
+
+            # and map them to colors
+            mapper = plt.cm.ScalarMappable(norm=norm, cmap=cmap)
+            colors = mapper.to_rgba(values)
+
+            if kwargs.pop("color", None) is not None:
+                logger = logging.getLogger(self.__class__.__name__)
+                logger.warning("`color` is overwritten by `color_value`.")
+
+        else:
+            colors = [kwargs.pop("color", None)] * len(self)
+
         # get patches representing all droplets
         if grid is None or not repeat_periodically:
             # plot only the droplets themselves
-            patches = [droplet._get_mpl_patch(**kwargs) for droplet in self]
+            patches = [
+                droplet._get_mpl_patch(dim=2, color=color, **kwargs)
+                for droplet, color in zip(self, colors)
+            ]
         else:
             # plot droplets also in their mirror positions
             patches = []
-            for droplet in self:
+            for droplet, color in zip(self, colors):
                 for p in grid.iter_mirror_points(
                     droplet.position, with_self=True, only_periodic=True
                 ):
                     # create copy with changed position
                     d = droplet.copy(position=p)
-                    patches.append(d._get_mpl_patch(**kwargs))
+                    patches.append(d._get_mpl_patch(dim=2, color=color, **kwargs))
 
         # add all patches as a collection
         import matplotlib as mpl
@@ -559,9 +626,21 @@ class Emulsion(list):
         coll = mpl.collections.PatchCollection(patches, match_original=True)
         ax.add_collection(coll)
 
+        # add colorbar if requested
+        if color_value is not None and colorbar:
+            from pde.tools.plotting import add_scaled_colorbar
+
+            label = colorbar if isinstance(colorbar, str) else ""
+            add_scaled_colorbar(mapper, ax=ax, label=label)
+
+        parameters = {
+            "repeat_periodicially": repeat_periodically,
+        }
+        return PlotReference(ax, coll, parameters)
+
 
 class EmulsionTimeCourse:
-    """ represents emulsions as a function of time """
+    """represents emulsions as a function of time"""
 
     def __init__(self, emulsions=None, times=None):
         """
@@ -597,7 +676,7 @@ class EmulsionTimeCourse:
 
     def append(
         self, emulsion: Emulsion, time: Optional[float] = None, copy: bool = True
-    ):
+    ) -> None:
         """add an emulsion to the list
 
         Args:
@@ -625,8 +704,8 @@ class EmulsionTimeCourse:
             time = 0 if len(self.times) == 0 else self.times[-1] + 1
         self.times.append(time)
 
-    def clear(self):
-        """ removes all data stored in this instance """
+    def clear(self) -> None:
+        """removes all data stored in this instance"""
         self.emulsions = []
         self.times = []
 
@@ -640,7 +719,7 @@ class EmulsionTimeCourse:
         return len(self.times)
 
     def __getitem__(self, key: Union[int, slice]):
-        """ return the information for the given index """
+        """return the information for the given index"""
         result = self.emulsions.__getitem__(key)
         if isinstance(key, slice):
             return self.__class__(emulsions=result, times=self.times[key])
@@ -648,15 +727,15 @@ class EmulsionTimeCourse:
             return result
 
     def __iter__(self) -> Iterator[Tuple[float, Emulsion]]:
-        """ iterate over the emulsions """
+        """iterate over the emulsions"""
         return iter(self.emulsions)
 
     def items(self):
-        """ iterate over all times and emulsions, returning them in pairs """
+        """iterate over all times and emulsions, returning them in pairs"""
         return zip(self.times, self.emulsions)
 
     def __eq__(self, other):
-        """ determine whether two EmulsionTimeCourse instance are equal """
+        """determine whether two EmulsionTimeCourse instance are equal"""
         grids_equal = self.grid is None or other.grid is None or self.grid == other.grid
         return (
             grids_equal
@@ -666,15 +745,19 @@ class EmulsionTimeCourse:
 
     @classmethod
     def from_storage(
-        cls, storage: StorageBase, progress: bool = True, **kwargs
+        cls, storage: StorageBase, refine: bool = False, progress: bool = None, **kwargs
     ) -> "EmulsionTimeCourse":
         r"""create an emulsion time course from a stored phase field
 
         Args:
             storage (:class:`~pde.storage.base.StorageBase`):
                 The phase fields for many time instances
+            refine (bool):
+                Flag determining whether the droplet properties should be refined
+                using fitting. This is a potentially slow procedure.
             progress (bool):
-                Whether to show the progress of the process
+                Whether to show the progress of the process. If `None`, the progress is
+                only shown when `refine` is `True`.
             \**kwargs:
                 All other parameters are forwarded to the
                 :meth:`~droplets.image_analysis.locate_droplets`.
@@ -684,21 +767,27 @@ class EmulsionTimeCourse:
         """
         from .image_analysis import locate_droplets
 
-        result = cls()
+        if progress is None:
+            progress = refine  # show progress only when refining by default
 
-        it = display_progress(storage.items(), total=len(storage), enabled=progress)
-        for t, frame in it:
-            emulsion = locate_droplets(frame, **kwargs)
-            result.append(emulsion, time=t)
-        return result
+        # obtain the emulsion data for all frames
+        emulsions = (
+            locate_droplets(frame, refine=refine, **kwargs)
+            for frame in display_progress(storage, enabled=progress)
+        )
+
+        return cls(emulsions, times=storage.times)
 
     @classmethod
-    def from_file(cls, filename: str, progress: bool = True) -> "EmulsionTimeCourse":
+    def from_file(cls, path: str, progress: bool = True) -> "EmulsionTimeCourse":
         """create emulsion time course by reading file
 
         Args:
-            filename (str): The filename from which the emulsion is read
-            progress (bool): Whether to show the progress of the process
+            path (str):
+                The path from which the data is read. This function assumes that the
+                data was written as an HDF5 file using :meth:`to_file`.
+            progress (bool):
+                Whether to show the progress of the process in a progress bar
 
         Returns:
             EmulsionTimeCourse: an instance describing the emulsion time course
@@ -706,7 +795,7 @@ class EmulsionTimeCourse:
         import h5py
 
         obj = cls()
-        with h5py.File(filename, "r") as fp:
+        with h5py.File(path, "r") as fp:
             # read grid
             if "grid" in fp.attrs:
                 grid: Optional[GridBase] = GridBase.from_state(fp.attrs["grid"])
@@ -724,16 +813,20 @@ class EmulsionTimeCourse:
                 )
         return obj
 
-    def to_file(self, filename: str, info: InfoDict = None):
+    def to_file(self, path: str, info: InfoDict = None) -> None:
         """store data in hdf5 file
 
+        The data can be read using the classmethod :meth:`EmulsionTimeCourse.from_file`.
+
         Args:
-            filename (str): determines the location where the file is written
-            info (dict): can be additional data stored alongside
+            path (str):
+                The path to which the data is written as an HDF5 file.
+            info (dict):
+                Additional data stored alongside the droplet track list
         """
         import h5py
 
-        with h5py.File(filename, "w") as fp:
+        with h5py.File(path, "w") as fp:
             # write the actual emulsion data
             for i, (time, emulsion) in enumerate(self.items()):
                 dataset = emulsion._write_hdf_dataset(fp, f"time_{i:06d}")
@@ -750,6 +843,7 @@ class EmulsionTimeCourse:
             if self.grid is not None:
                 fp.attrs["grid"] = self.grid.state_serialized
 
+    @fill_in_docstring
     def tracker(
         self,
         interval: Union[int, float, IntervalType] = 1,
@@ -758,12 +852,8 @@ class EmulsionTimeCourse:
         """return a tracker that analyzes emulsions during simulations
 
         Args:
-            interval: Determines how often the tracker interrupts the
-                simulation. Simple numbers are interpreted as durations measured
-                in the simulation time variable. Alternatively, instances of
-                :class:`~droplets.simulation.trackers.LogarithmicIntervals` and
-                :class:`~droplets.simulation.trackers.RealtimeIntervals` might
-                be given for more control.
+            interval:
+                {ARG_TRACKER_INTERVAL}
             filename (str): determines where the EmulsionTimeCourse data is
                 stored
         """
