@@ -31,11 +31,12 @@ except ImportError:
     from numpy.fft import fftn as np_fftn
 
 from pde.fields import ScalarField
-from pde.grids import CylindricalGrid
+from pde.grids import CylindricalSymGrid
 from pde.grids.base import GridBase
-from pde.grids.cartesian import CartesianGridBase
-from pde.grids.spherical import SphericalGridBase
+from pde.grids.cartesian import CartesianGrid
+from pde.grids.spherical import SphericalSymGridBase
 from pde.tools.math import SmoothData1D
+from pde.tools.typing import NumberOrArray
 
 from .droplets import (
     DiffuseDroplet,
@@ -47,14 +48,14 @@ from .emulsions import Emulsion
 
 
 def _locate_droplets_in_mask_cartesian(
-    grid: CartesianGridBase, mask: np.ndarray
+    grid: CartesianGrid, mask: np.ndarray
 ) -> Emulsion:
     """locate droplets in a (potentially periodic) data set on a Cartesian grid
 
     This function locates droplets respecting periodic boundary conditions.
 
     Args:
-        mask (:class:`numpy.ndarray`):
+        mask (:class:`~numpy.ndarray`):
             The binary image (or mask) in which the droplets are searched
 
     Returns:
@@ -68,7 +69,7 @@ def _locate_droplets_in_mask_cartesian(
 
     # pad the array to simulate periodic boundary conditions
     offset = np.array([dim if p else 0 for p, dim in zip(grid.periodic, grid.shape)])
-    pad = np.c_[offset, offset].astype(np.int)
+    pad = np.c_[offset, offset].astype(np.intc)
     mask_padded = np.pad(mask, pad, mode="wrap")
     assert np.all(mask_padded.shape == np.array(grid.shape) + 2 * offset)
 
@@ -79,12 +80,12 @@ def _locate_droplets_in_mask_cartesian(
     indices = range(1, num_labels + 1)
 
     # create and emulsion from this of droplets
-    grid._logger.info(f"Found {num_labels} droplet candidates")
+    grid._logger.info(f"Found {num_labels} droplet candidate(s)")
 
     # determine position from binary image and scale it to real space
     positions = ndimage.measurements.center_of_mass(mask_padded, labels, index=indices)
     # correct for the additional padding of the array
-    positions = grid.cell_to_point(positions) - offset
+    positions = grid.transform(positions - offset, "cell", "grid")
 
     # determine volume from binary image and scale it to real space
     volumes = ndimage.measurements.sum(mask_padded, labels, index=indices)
@@ -99,18 +100,24 @@ def _locate_droplets_in_mask_cartesian(
 
     # filter overlapping droplets (e.g. due to duplicates)
     emulsion = Emulsion(droplets, grid=grid)
+    num_candidates = len(emulsion)
+    if num_candidates < num_labels:
+        grid._logger.info(f"Only {num_candidates} candidate(s) inside bounds")
+
     emulsion.remove_overlapping()
+    if len(emulsion) < num_candidates:
+        grid._logger.info(f"Only {num_candidates} candidate(s) not overlapping")
 
     return emulsion
 
 
 def _locate_droplets_in_mask_spherical(
-    grid: SphericalGridBase, mask: np.ndarray
+    grid: SphericalSymGridBase, mask: np.ndarray
 ) -> Emulsion:
     """locates droplets in a binary data set on a spherical grid
 
     Args:
-        mask (:class:`numpy.ndarray`):
+        mask (:class:`~numpy.ndarray`):
             The binary image (or mask) in which the droplets are searched
 
     Returns:
@@ -128,8 +135,7 @@ def _locate_droplets_in_mask_spherical(
     droplet = None
     for slices in object_slices:
         if slices[0].start == 0:  # contains point around origin
-            radius = grid.cell_to_point(slices[0].stop).flat[-1]
-            radius += grid.axes_bounds[0][0]  # add inner radius
+            radius = float(grid.transform(slices[0].stop, "cell", "grid").flat[-1])
             droplet = SphericalDroplet(np.zeros(grid.dim), radius=radius)
         else:
             logger = logging.getLogger(grid.__class__.__module__)
@@ -143,12 +149,12 @@ def _locate_droplets_in_mask_spherical(
 
 
 def _locate_droplets_in_mask_cylindrical_single(
-    grid: CylindricalGrid, mask: np.ndarray
+    grid: CylindricalSymGrid, mask: np.ndarray
 ) -> Emulsion:
     """locate droplets in a data set on a single cylindrical grid
 
     Args:
-        mask (:class:`numpy.ndarray`):
+        mask (:class:`~numpy.ndarray`):
             The binary image (or mask) in which the droplets are searched
 
     Returns:
@@ -171,7 +177,7 @@ def _locate_droplets_in_mask_cylindrical_single(
 
     # determine position from binary image and scale it to real space
     pos = ndimage.measurements.center_of_mass(mask, labels, index=indices)
-    pos = grid.cell_to_point(pos)
+    pos = grid.transform(pos, "cell", "cartesian")
 
     # determine volume from binary image and scale it to real space
     vol_r, dz = grid.cell_volume_data
@@ -179,19 +185,22 @@ def _locate_droplets_in_mask_cylindrical_single(
     vol = ndimage.measurements.sum(cell_volumes, labels, index=indices)
 
     # return an emulsion of droplets
-    droplets = (SphericalDroplet.from_volume((0, 0, p[2]), v) for p, v in zip(pos, vol))
+    droplets = (
+        SphericalDroplet.from_volume(np.array([0, 0, p[2]]), v)
+        for p, v in zip(pos, vol)
+    )
     return Emulsion(droplets, grid=grid)
 
 
 def _locate_droplets_in_mask_cylindrical(
-    grid: CylindricalGrid, mask: np.ndarray
+    grid: CylindricalSymGrid, mask: np.ndarray
 ) -> Emulsion:
     """locate droplets in a data set on a (periodic) cylindrical grid
 
     This function locates droplets respecting periodic boundary conditions.
 
     Args:
-        mask (:class:`numpy.ndarray`):
+        mask (:class:`~numpy.ndarray`):
             The binary image (or mask) in which the droplets are searched
 
     Returns:
@@ -204,6 +213,7 @@ def _locate_droplets_in_mask_cylindrical(
 
         # pad the array to simulate periodic boundary conditions
         dim_r, dim_z = grid.shape
+        z_min, z_max = grid.axes_bounds[1]
         mask_padded = np.pad(mask, [[0, 0], [dim_z, dim_z]], mode="wrap")
         assert mask_padded.shape == (dim_r, 3 * dim_z)
 
@@ -217,7 +227,7 @@ def _locate_droplets_in_mask_cylindrical(
             # correct for the additional padding of the array
             droplet.position[2] -= grid.length
             # check whether the droplet lies in the original box
-            if grid.contains_point(droplet.position):
+            if z_min <= droplet.position[2] <= z_max:
                 droplets.append(droplet)
 
         grid._logger.info(f"Kept {len(droplets)} central droplets.")
@@ -238,17 +248,17 @@ def locate_droplets_in_mask(grid: GridBase, mask: np.ndarray) -> Emulsion:
     This function locates droplets respecting periodic boundary conditions.
 
     Args:
-        mask (:class:`numpy.ndarray`):
+        mask (:class:`~numpy.ndarray`):
             The binary image (or mask) in which the droplets are searched
 
     Returns:
         :class:`droplets.emulsions.Emulsion`: The discovered spherical droplets
     """
-    if isinstance(grid, CartesianGridBase):
+    if isinstance(grid, CartesianGrid):
         return _locate_droplets_in_mask_cartesian(grid, mask)
-    elif isinstance(grid, SphericalGridBase):
+    elif isinstance(grid, SphericalSymGridBase):
         return _locate_droplets_in_mask_spherical(grid, mask)
-    elif isinstance(grid, CylindricalGrid):
+    elif isinstance(grid, CylindricalSymGrid):
         return _locate_droplets_in_mask_cylindrical(grid, mask)
     elif isinstance(grid, GridBase):
         raise NotImplementedError(f"Locating droplets is not possible for grid {grid}")
@@ -263,7 +273,7 @@ def locate_droplets(
     minimal_radius: float = 0,
     refine: bool = False,
     interface_width: Optional[float] = None,
-):
+) -> Emulsion:
     """Locates droplets in the phase field
 
     This uses a binarized image to locate clusters of large concentration in the
@@ -292,8 +302,7 @@ def locate_droplets(
             starting value for the fitting if droplets are refined.
 
     Returns:
-        :class:`~pde.analysis.emulsions.Emulsion`:
-        An emulsion containing all detected droplets
+        :class:`~droplets.emulsions.Emulsion`: All detected droplets
     """
     assert isinstance(phase_field, ScalarField)
     dim = phase_field.grid.dim  # dimensionality of the space
@@ -303,7 +312,7 @@ def locate_droplets(
 
     # determine actual threshold
     if threshold == "auto":
-        threshold = (phase_field.data.min() + phase_field.data.max()) / 2
+        threshold = float(phase_field.data.min() + phase_field.data.max()) / 2
     else:
         threshold = float(threshold)
 
@@ -318,7 +327,7 @@ def locate_droplets(
     for droplet in candidates:
         # check whether we need to add the interface width
         droplet_class = droplet.__class__
-        args = {}
+        args: Dict[str, NumberOrArray] = {}
 
         # change droplet class when interface width is given
         if interface_width is not None:
@@ -358,7 +367,7 @@ def refine_droplet(
     phase_field: ScalarField,
     droplet: DiffuseDroplet,
     least_squares_params: Optional[Dict[str, Any]] = None,
-):
+) -> DiffuseDroplet:
     """Refines droplet parameters by fitting to phase field
 
     This function varies droplet parameters, like position, size,
@@ -377,7 +386,8 @@ def refine_droplet(
             documentation of scipy.optimize.least_squares
 
     Returns:
-        The refined droplet as an instance of the argument `droplet`
+        :class:`droplets.droplets.DiffuseDroplet`: The refined droplet as an instance of
+        the argument `droplet`
     """
     assert isinstance(phase_field, ScalarField)
     if least_squares_params is None:
@@ -389,9 +399,9 @@ def refine_droplet(
         droplet.interface_width = phase_field.grid.typical_discretization
 
     # enlarge the mask to also contain the shape change
-    mask = droplet.get_phase_field(phase_field.grid, dtype=np.bool)
+    mask = droplet._get_phase_field(phase_field.grid, dtype=np.bool_)
     dilation_iterations = 1 + int(2 * droplet.interface_width)
-    mask = ndimage.morphology.binary_dilation(mask.data, iterations=dilation_iterations)
+    mask = ndimage.morphology.binary_dilation(mask, iterations=dilation_iterations)
 
     # apply the mask
     data_mask = phase_field.data[mask]
@@ -399,7 +409,7 @@ def refine_droplet(
     # determine the coordinate constraints and only vary the free data points
     data_flat = structured_to_unstructured(droplet.data)  # unstructured data
     dtype = droplet.data.dtype
-    free = np.ones(len(data_flat), dtype=np.bool)
+    free = np.ones(len(data_flat), dtype=np.bool_)
     free[phase_field.grid.coordinate_constraints] = False
 
     # determine data bounds
@@ -407,12 +417,12 @@ def refine_droplet(
     bounds = l[free], h[free]
 
     def _image_deviation(params):
-        """ helper function evaluating the residuals """
+        """helper function evaluating the residuals"""
         # generate the droplet
         data_flat[free] = params
         droplet.data = unstructured_to_structured(data_flat, dtype=dtype)
         droplet.check_data()
-        img = droplet.get_phase_field_masked(phase_field.grid, mask=mask)
+        img = droplet._get_phase_field(phase_field.grid)[mask]
         return img - data_mask
 
     # do the least square optimization
@@ -423,7 +433,10 @@ def refine_droplet(
     droplet.data = unstructured_to_structured(data_flat, dtype=dtype)
 
     # normalize the droplet position
-    droplet.position = phase_field.grid.normalize_point(droplet.position)
+    grid = phase_field.grid
+    coords = grid.transform(droplet.position, "cartesian", "grid")
+    droplet.position = grid.transform(grid.normalize_point(coords), "grid", "cartesian")
+
     return droplet
 
 
@@ -468,7 +481,7 @@ def get_structure_factor(
         )
 
     grid = scalar_field.grid
-    if not isinstance(grid, CartesianGridBase):
+    if not isinstance(grid, CartesianGrid):
         raise NotImplementedError(
             "Structure factor can currently only be calculated for Cartesian grids"
         )
@@ -533,36 +546,36 @@ def get_structure_factor(
 
 
 def get_length_scale(
-    scalar_field: ScalarField,
-    method: str = "structure_factor_maximum",
-    full_output: bool = False,
-    smoothing: Optional[float] = None,
-):
+    scalar_field: ScalarField, method: str = "structure_factor_maximum", **kwargs
+) -> Union[float, Tuple[float, Any]]:
     """Calculates a length scale associated with a phase field
 
     Args:
         scalar_field (:class:`~pde.fields.ScalarField`):
-            The scalar_field being analyzed
+            The scala field being analyzed
         method (str):
-            A string determining which method is used to calculate the length
-            scale.Valid options are `structure_factor_maximum` (numerically
-            determine the maximum in the structure factor) and
-            `structure_factor_mean` (calculate the mean of the structure
-            factor).
-        full_output (bool):
-            Flag determining whether additional data is returned. The format of
-            the returned data depends on the method.
-        smoothing (float, optional):
-            Length scale that determines the smoothing of the radially averaged
-            structure factor. If `None` it is automatically determined from the
-            typical discretization of the underlying grid. This parameter is
-            only used if `method = 'structure_factor_maximum'`
+            A string determining which method is used to calculate the length scale.
+            Valid options are `structure_factor_maximum` (numerically determine the
+            maximum in the structure factor) and `structure_factor_mean` (calculate the
+            mean of the structure factor).
+
+    Additional supported keyword arguments depend on the chosen method. For instance,
+    the methods involving the structure factor allow for a boolean flag `full_output`,
+    which also returns the actual structure factor. The method
+    `structure_factor_maximum` also allows for some smoothing of the radially averaged
+    structure factor. If the parameter `smoothing` is set to `None` the amount of
+    smoothing is determined automatically from the typical discretization of the
+    underlying grid. For the method `droplet_detection`, additional arguments are
+    forwarded to :func:`locate_droplets`.
 
     Returns:
         float: The determine length scale
 
         If `full_output = True`, a tuple with the length scale and an additional
         object with further information is returned.
+
+    See Also:
+        :class:`~droplets.trackers.LengthScaleTracker`: Tracker measuring length scales
     """
     logger = logging.getLogger(__name__)
 
@@ -571,7 +584,7 @@ def get_length_scale(
         k_mag, sf = get_structure_factor(scalar_field)
         length_scale = np.sum(sf) / np.sum(k_mag * sf)
 
-        if full_output:
+        if kwargs.pop("full_output", False):
             return length_scale, sf
 
     elif method == "structure_factor_maximum" or method == "structure_factor_peak":
@@ -579,7 +592,7 @@ def get_length_scale(
         k_mag, sf = get_structure_factor(scalar_field, smoothing=None)
 
         # smooth the structure factor
-        if smoothing is None:
+        if kwargs.pop("smoothing", None) is None:
             smoothing = 0.01 * scalar_field.grid.typical_discretization
         sf_smooth = SmoothData1D(k_mag, sf, sigma=smoothing)
 
@@ -605,17 +618,36 @@ def get_length_scale(
                     )
                 length_scale = 1 / result.x
 
-        if full_output:
+        if kwargs.pop("full_output", False):
             return length_scale, sf_smooth
 
+    elif method == "droplet_detection":
+        # calculate the length scale from detected droplets
+        droplets = locate_droplets(scalar_field, **kwargs)
+        kwargs = None  # clear kwargs, so no warning is raised
+
+        # get the axes along which droplets can be placed
+        grid = scalar_field.grid
+        axes = set(range(grid.dim)) - set(grid.coordinate_constraints)
+        volume = 1
+        for ax in axes:
+            volume *= grid.axes_bounds[ax][1] - grid.axes_bounds[ax][0]
+
+        volume_per_droplet = volume / len(droplets)
+        length_scale = volume_per_droplet ** (1 / len(axes))
+
     else:
         raise ValueError(
             f"Method {method} is not defined. Valid values are `structure_factor_mean` "
             "and `structure_factor_maximum`"
         )
 
+    if kwargs:
+        # raise warning if keyword arguments remain
+        logger.warning("Unused keyword arguments: %s", ", ".join(kwargs))
+
     # return only the length scale with out any additional information
-    return length_scale
+    return length_scale  # type: ignore
 
 
 __all__ = [
