@@ -6,24 +6,24 @@ import numpy as np
 import pytest
 from numpy.lib.recfunctions import structured_to_unstructured
 
-from pde import (
+from pde import ScalarField
+from pde.grids import (
     CartesianGrid,
-    CylindricalGrid,
-    PolarGrid,
-    ScalarField,
-    SphericalGrid,
+    CylindricalSymGrid,
+    PolarSymGrid,
+    SphericalSymGrid,
     UnitGrid,
 )
 
-from .. import image_analysis
-from ..droplets import DiffuseDroplet, PerturbedDroplet2D, PerturbedDroplet3D
-from ..emulsions import Emulsion
+from droplets import image_analysis
+from droplets.droplets import DiffuseDroplet, PerturbedDroplet2D, PerturbedDroplet3D
+from droplets.emulsions import Emulsion
 
 
 @pytest.mark.parametrize("size", [16, 17])
 @pytest.mark.parametrize("periodic", [True, False])
 def test_localization_sym_unit(size, periodic):
-    """ tests simple droplets localization in 2d """
+    """tests simple droplets localization in 2d"""
     pos = np.random.random(2) * size
     radius = np.random.uniform(2, 5)
     width = np.random.uniform(1, 2)
@@ -49,7 +49,7 @@ def test_localization_sym_unit(size, periodic):
 
 @pytest.mark.parametrize("periodic", [True, False])
 def test_localization_sym_rect(periodic):
-    """ tests simple droplets localization in 2d with a rectangular grid"""
+    """tests simple droplets localization in 2d with a rectangular grid"""
     size = 16
 
     pos = np.random.uniform(-4, 4, size=2)
@@ -59,7 +59,7 @@ def test_localization_sym_rect(periodic):
 
     a = np.random.random(2) - size / 2
     b = np.random.random(2) + size / 2
-    grid = CartesianGrid(np.c_[a, b], size, periodic=periodic)
+    grid = CartesianGrid(np.c_[a, b], 3 * size, periodic=periodic)
     field = d1.get_phase_field(grid)
 
     emulsion = image_analysis.locate_droplets(field, refine=True)
@@ -76,13 +76,13 @@ def test_localization_sym_rect(periodic):
 
 @pytest.mark.parametrize("periodic", [True, False])
 def test_localization_perturbed_2d(periodic):
-    """ tests localization of perturbed 2d droplets """
+    """tests localization of perturbed 2d droplets"""
     size = 16
 
     pos = np.random.uniform(-4, 4, size=2)
     radius = np.random.uniform(2, 5)
     width = np.random.uniform(0.5, 1.5)
-    ampls = np.random.uniform(-0.01, 0.01, size=3)
+    ampls = np.random.uniform(-0.01, 0.01, size=4)
     d1 = PerturbedDroplet2D(pos, radius, interface_width=width, amplitudes=ampls)
 
     a = np.random.random(2) - size / 2
@@ -103,13 +103,13 @@ def test_localization_perturbed_2d(periodic):
 
 @pytest.mark.parametrize("periodic", [True, False])
 def test_localization_perturbed_3d(periodic):
-    """ tests localization of perturbed 3d droplets """
+    """tests localization of perturbed 3d droplets"""
     size = 8
 
     pos = np.random.uniform(-2, 2, size=3)
     radius = np.random.uniform(2, 3)
     width = np.random.uniform(0.5, 1.5)
-    ampls = np.random.uniform(-0.01, 0.01, size=7)
+    ampls = np.random.uniform(-0.01, 0.01, size=3)
     d1 = PerturbedDroplet3D(pos, radius, interface_width=width, amplitudes=ampls)
 
     a = np.random.random(3) - size / 2
@@ -124,7 +124,7 @@ def test_localization_perturbed_3d(periodic):
 
     msg = "size=%d, periodic=%s, %s != %s" % (size, periodic, d1, d2)
     np.testing.assert_almost_equal(d1.position, d2.position, decimal=1, err_msg=msg)
-    assert d1.radius == pytest.approx(d2.radius, rel=1e-5)
+    assert d1.radius == pytest.approx(d2.radius, rel=1e-4)
     assert d1.interface_width == pytest.approx(d2.interface_width, rel=1e-3)
     np.testing.assert_allclose(
         d1.amplitudes[3:], d2.amplitudes[3:], rtol=0.5, err_msg=msg
@@ -132,13 +132,13 @@ def test_localization_perturbed_3d(periodic):
 
 
 def test_localization_polar():
-    """ tests simple droplets localization in polar grid """
+    """tests simple droplets localization in polar grid"""
     radius = np.random.uniform(2, 3)
     width = np.random.uniform(0.5, 1.5)
     d1 = DiffuseDroplet((0, 0), radius, interface_width=width)
 
     grid_radius = 6 + 2 * np.random.random()
-    grid = PolarGrid(grid_radius, 16)
+    grid = PolarSymGrid(grid_radius, 16)
     field = d1.get_phase_field(grid)
 
     emulsion = image_analysis.locate_droplets(field, refine=True)
@@ -154,13 +154,13 @@ def test_localization_polar():
 
 
 def test_localization_spherical():
-    """ tests simple droplets localization in spherical grid """
+    """tests simple droplets localization in spherical grid"""
     radius = np.random.uniform(2, 3)
     width = np.random.uniform(0.5, 1.5)
     d1 = DiffuseDroplet((0, 0, 0), radius, interface_width=width)
 
     grid_radius = 6 + 2 * np.random.random()
-    grid = SphericalGrid(grid_radius, 16)
+    grid = SphericalSymGrid(grid_radius, 16)
     field = d1.get_phase_field(grid)
 
     emulsion = image_analysis.locate_droplets(field, refine=True)
@@ -177,7 +177,7 @@ def test_localization_spherical():
 
 @pytest.mark.parametrize("periodic", [True, False])
 def test_localization_cylindrical(periodic):
-    """ tests simple droplets localization in cylindrical grid """
+    """tests simple droplets localization in cylindrical grid"""
     pos = (0, 0, np.random.uniform(-4, 4))
     radius = np.random.uniform(2, 3)
     width = np.random.uniform(0.5, 1.5)
@@ -185,7 +185,7 @@ def test_localization_cylindrical(periodic):
 
     grid_radius = 6 + 2 * np.random.random()
     bounds_z = np.random.uniform(1, 2, size=2) * np.array([-4, 4])
-    grid = CylindricalGrid(grid_radius, bounds_z, (16, 32), periodic_z=periodic)
+    grid = CylindricalSymGrid(grid_radius, bounds_z, (16, 32), periodic_z=periodic)
     field = d1.get_phase_field(grid)
 
     emulsion = image_analysis.locate_droplets(field, refine=True)
@@ -201,7 +201,7 @@ def test_localization_cylindrical(periodic):
 
 
 def test_localization_threshold():
-    """ tests different localization thresholds """
+    """tests different localization thresholds"""
     pos = np.random.random(2) * 16
     radius = np.random.uniform(2, 5)
     width = np.random.uniform(1, 2)
@@ -223,16 +223,20 @@ def test_localization_threshold():
 
 
 def test_get_length_scale():
-    """ test determining the length scale """
+    """test determining the length scale"""
     grid = CartesianGrid([[0, 8 * np.pi]], 64, periodic=True)
     c = ScalarField(grid, np.sin(grid.axes_coords[0]))
-    for method in ["structure_factor_mean", "structure_factor_maximum"]:
+    for method in [
+        "structure_factor_mean",
+        "structure_factor_maximum",
+        "droplet_detection",
+    ]:
         s = image_analysis.get_length_scale(c, method=method)
         assert s == pytest.approx(2 * np.pi, rel=0.1)
 
 
 def test_emulsion_processing():
-    """ test identifying emulsions in phase fields """
+    """test identifying emulsions in phase fields"""
     grid = UnitGrid([32, 32], periodic=True)
 
     e1 = Emulsion(
@@ -254,7 +258,7 @@ def test_emulsion_processing():
 
 
 def test_structure_factor_random():
-    """ test the structure factor function for random input """
+    """test the structure factor function for random input"""
     g1 = CartesianGrid([[0, 10]] * 2, 64, periodic=True)
     f1 = ScalarField.random_colored(g1, -2)
 
