@@ -15,6 +15,7 @@ from typing import Callable, List, Optional, Union  # @UnusedImport
 import numpy as np
 
 from pde.fields.base import FieldBase
+from pde.tools.docstrings import fill_in_docstring
 from pde.trackers.base import InfoDict, TrackerBase
 from pde.trackers.intervals import IntervalData
 
@@ -31,10 +32,12 @@ class LengthScaleTracker(TrackerBase):
             The associated length scales
     """
 
+    @fill_in_docstring
     def __init__(
         self,
         interval: IntervalData = 1,
         filename: Optional[str] = None,
+        *,
         method: str = "structure_factor_mean",
         source: Union[None, int, Callable] = None,
         verbose: bool = False,
@@ -42,24 +45,23 @@ class LengthScaleTracker(TrackerBase):
         r"""
         Args:
             interval:
-                |Arg_tracker_interval|
+                {ARG_TRACKER_INTERVAL}
             filename (str, optional):
                 Determines the file to which the data is written in JSON format
             method (str):
-                Method used for determining the length scale. Methods are
-                explain in the function :func:`~pde.analysis.get\_length\_scale`
+                Method used for determining the length scale. Details are explained in
+                the function :func:`~droplets.image_analysis.get_length_scale`.
             source (int or callable, optional):
-                Determines how a field is extracted from `fields`. If `None`,
-                `fields` is passed as is, assuming it is already a scalar field.
-                This works for the simple, standard case where only a single
-                ScalarField is treated. Alternatively, `source` can be an
-                integer, indicating which field is extracted from an instance of
-                :class:`~pde.fields.FieldCollection`. Lastly, `source` can be a
-                function that takes `fields` as an argument and returns the
-                desired field.
+                Determines how a field is extracted from `fields`. If `None`, `fields`
+                is passed as is, assuming it is already a scalar field. This works for
+                the simple, standard case where only a single
+                :class:`~pde.fields.scalar.ScalarField` is treated. Alternatively,
+                `source` can be an integer, indicating which field is extracted from an
+                instance of :class:`~pde.fields.collection.FieldCollection`. Lastly,
+                `source` can be a function that takes `fields` as an argument and
+                returns the desired field.
             verbose (bool):
-                Determines whether errors in determining the length scales are
-                logged.
+                Determines whether errors in determining the length scales are logged.
         """
         super().__init__(interval=interval)
         self.length_scales: List[float] = []
@@ -94,7 +96,7 @@ class LengthScaleTracker(TrackerBase):
 
         # store data
         self.times.append(t)
-        self.length_scales.append(length)
+        self.length_scales.append(length)  # type: ignore
 
     def finalize(self, info: InfoDict = None) -> None:
         """finalize the tracker, supplying additional information
@@ -125,12 +127,15 @@ class DropletTracker(TrackerBase):
             done.
     """
 
+    @fill_in_docstring
     def __init__(
         self,
         interval: IntervalData = 1,
         filename: Optional[str] = None,
+        *,
         emulsion_timecourse=None,
         source: Union[None, int, Callable] = None,
+        threshold: Union[float, str] = 0.5,
         minimal_radius: float = 0,
         refine: bool = False,
         perturbation_modes: int = 0,
@@ -138,22 +143,23 @@ class DropletTracker(TrackerBase):
         """
         Args:
             interval:
-                |Arg_tracker_interval|
+                {ARG_TRACKER_INTERVAL}
             filename (str, optional):
-                Determines the file to which the final data is written.
+                Determines the file to which the final data is written as an HDF5 file.
             emulsion_timecourse (:class:`EmulsionTimeCourse`, optional):
-                Can be an instance of
-                :class:`~droplets.emulsions.EmulsionTimeCourse` that is
-                used to store the data.
+                Can be an instance of :class:`~droplets.emulsions.EmulsionTimeCourse`
+                that is used to store the data.
             source (int or callable, optional):
-                Determines how a field is extracted from `fields`. If `None`,
-                `fields` is passed as is, assuming it is already a scalar field.
-                This works for the simple, standard case where only a single
-                ScalarField is treated. Alternatively, `source` can be an
-                integer, indicating which field is extracted from an instance of
-                :class:`~pde.fields.FieldCollection`. Lastly, `source` can be a
-                function that takes `fields` as an argument and returns the
-                desired field.
+                Determines how a field is extracted from `fields`. If `None`, `fields`
+                is passed as is, assuming it is already a scalar field. This works for
+                the simple, standard case where only a single ScalarField is treated.
+                Alternatively, `source` can be an integer, indicating which field is
+                extracted from an instance of :class:`~pde.fields.FieldCollection`.
+                Lastly, `source` can be a function that takes `fields` as an argument
+                and returns the desired field.
+            threshold (float or str):
+                The threshold for binarizing the image. The special value 'auto' takes
+                the mean between the minimum and the maximum of the data as a guess.
             minimal_radius (float):
                 Minimal radius of droplets that will be retained.
             refine (bool):
@@ -171,6 +177,7 @@ class DropletTracker(TrackerBase):
             self.data = emulsion_timecourse
         self.filename = filename
         self.source = source
+        self.threshold = threshold
         self.minimal_radius = minimal_radius
         self.refine = refine
         self.perturbation_modes = perturbation_modes
@@ -211,9 +218,10 @@ class DropletTracker(TrackerBase):
         scalar_field = extract_field(field, self.source, 0)
         emulsion = locate_droplets(
             scalar_field,  # type: ignore
-            minimal_radius=self.minimal_radius,
+            threshold=self.threshold,
             refine=self.refine,
             modes=self.perturbation_modes,
+            minimal_radius=self.minimal_radius,
         )
         self.data.append(emulsion, t)
 
